"""
A collection of tools to read and process football data
from various sources.
"""


__all__ = ['FiveThirtyEight', 'ClubElo', 'MatchHistory']

from .FiveThirtyEight import FiveThirtyEight
from .ClubElo import ClubElo
from .MatchHistory import MatchHistory
